/*
 * Copyright (C) 2002 by MyVietnam.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding mvnForum
 * must remain intact in the scripts and in the outputted HTML
 * The "powered by" text/logo with a link back to
 * http://www.mvnForum.com and http://www.MyVietnam.net in the footer of the pages MUST
 * remain visible when the pages are viewed on the internet or intranet.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Support can be obtained from support forums at:
 * http://www.mvnForum.com/mvnforum/index
 *
 * Correspondence and Marketing Questions can be sent to:
 * info@MyVietnam.net
 *
 * @author: Minh Nguyen  minhnn@MyVietnam.net
 * @author: Mai  Nguyen  mai.nh@MyVietnam.net
 */
package net.myvietnam.mvnplugin.mvnforum.admin;

import javax.servlet.http.*;
import java.sql.*;
import java.util.Collection;
import net.myvietnam.mvncore.exception.*;
import net.myvietnam.mvncore.util.ParamUtil;
import net.myvietnam.mvncore.util.DateUtil;
import net.myvietnam.mvnplugin.mvnforum.db.CategoryBean;
import net.myvietnam.mvnplugin.mvnforum.auth.*;

class CategoryWebHandler {

    private OnlineUserManager onlineUserManager = OnlineUserManager.getInstance();

    CategoryWebHandler() {
    }

    /**
     * @todo: check permission
     */
    void processAdd(HttpServletRequest request)
        throws BadInputException, CreateException, DatabaseException, DuplicateKeyException,
        ForeignKeyNotFoundException, AuthenticationException, AssertionException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();
        permission.ensureCanNewCategory();

        Timestamp now = DateUtil.getCurrentGMTTimestamp();

        int parentCategoryID            = 0;
        String categoryName             = ParamUtil.getParameterSafe(request, "CategoryName", true);
        String categoryDesc             = ParamUtil.getParameterSafe(request, "CategoryDesc", false);
        int categoryOption              = 0;//xem lai
        int categoryStatus              = 0;//xem lai

        CategoryWebHelper.createCategory(parentCategoryID, categoryName, categoryDesc,
                                 now/*categoryCreationDate*/, now/*categoryModifiedDate*/, 0/*categoryOrder*/,
                                 categoryOption, categoryStatus);

        request.setAttribute("CategoryName", categoryName);
    }

    /**
     * @todo: check permission
     */
    public void prepareEdit(HttpServletRequest request)
        throws BadInputException, DatabaseException, AuthenticationException, AssertionException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();
        permission.ensureCanEditCategory();

        // primary key column(s)
        int categoryID  = ParamUtil.getParameterInt(request, "category");

        CategoryBean bean = CategoryWebHelper.getCategory(categoryID);

        request.setAttribute("CategoryBean", bean);
    }

    /**
     * @todo: check permission
     */
    public void processUpdate(HttpServletRequest request)
        throws BadInputException, DatabaseException, DuplicateKeyException,
        AuthenticationException, AssertionException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();
        permission.ensureCanEditCategory();

        Timestamp now = DateUtil.getCurrentGMTTimestamp();

        // primary key column(s)
        int categoryID                  = ParamUtil.getParameterInt(request, "CategoryID");

        // column(s) to update
        String categoryName             = ParamUtil.getParameterSafe(request, "CategoryName", true);
        String categoryDesc             = ParamUtil.getParameterSafe(request, "CategoryDesc", false);
        int categoryOrder               = ParamUtil.getParameterInt(request, "CategoryOrder");
        if (categoryOrder < 0) {
            throw new BadInputException("CategoryOrder cannot be negative");
        }
        int categoryOption              = 0;// xem lai
        int categoryStatus              = 0;// xem lai

        CategoryWebHelper.updateCategory(categoryID, // primary key
                                 categoryName, categoryDesc, now/*categoryModifiedDate*/,
                                 categoryOrder, categoryOption, categoryStatus);
    }

    /**
     * @todo: check permission
     */
    public void processUpdateCategoryOrder(HttpServletRequest request)
        throws BadInputException, DatabaseException, DuplicateKeyException,
        AuthenticationException, AssertionException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();
        permission.ensureCanEditCategory();

        Timestamp now = DateUtil.getCurrentGMTTimestamp();

        // primary key column(s)
        int categoryID                  = ParamUtil.getParameterInt(request, "category");

        String action                   = ParamUtil.getParameterSafe(request, "action", true);
        if (action.equals("up")) {
            CategoryWebHelper.decreaseCategoryOrder(categoryID, now);
        } else if (action.equals("down")) {
            CategoryWebHelper.increaseCategoryOrder(categoryID, now);
        } else {
            throw new BadInputException("Cannot update CategoryOrder: unknown action: " + action);
        }
    }

    /**
     * @todo: check permission
     */
    public void prepareList(HttpServletRequest request)
        throws DatabaseException, AssertionException, AuthenticationException {

        OnlineUser onlineUser = onlineUserManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();
        /** @todo is this a correct permission ??? */
        permission.ensureCanEditCategory();

        Collection beans = CategoryWebHelper.getCategories();

        request.setAttribute("CategoryBeans", beans);
    }


}
