/*
 * Copyright (C) 2002 by MyVietnam.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding mvnForum
 * must remain intact in the scripts and in the outputted HTML
 * The "powered by" text/logo with a link back to
 * http://www.mvnForum.com and http://www.MyVietnam.net in the footer of the pages MUST
 * remain visible when the pages are viewed on the internet or intranet.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Support can be obtained from support forums at:
 * http://www.mvnForum.com/mvnforum/index
 *
 * Correspondence and Marketing Questions can be sent to:
 * info@MyVietnam.net
 *
 * @author: Minh Nguyen  minhnn@MyVietnam.net
 * @author: Mai  Nguyen  mai.nh@MyVietnam.net
 */
package net.myvietnam.mvnplugin.mvnforum.auth;

import java.sql.Timestamp;
import net.myvietnam.mvncore.util.DateUtil;

public class OnlineUserAction {

    private String url;
    private String desc;
    private Timestamp firstRequestTime;
    private Timestamp lastRequestTime;

    // NOTE: these 2 variables seam to duplicate variables in OnlineUserImpl
    // but we need it when display all online users (listonlineuser)
    private int memberID;
    private String memberName;

    /**
     * default constructor
     */
    OnlineUserAction() {
        /*
         * We must separete 2 duplicate method calls since we will
         * update the lastRequestTime, but not firstRequestTime
         */
        firstRequestTime = DateUtil.getCurrentGMTTimestamp();
        lastRequestTime  = DateUtil.getCurrentGMTTimestamp();
    }

/****************************************************************
 * Public method
 ****************************************************************/

    public String getDesc() {
        return desc;
    }

    public String getUrl() {
        return url;
    }

    public int getMemberID() {
        return memberID;
    }

    public String getMemberName() {
        return memberName;
    }

    public Timestamp getFirstRequestTime() {
        return firstRequestTime;
    }

    public Timestamp getLastRequestTime() {
        return lastRequestTime;
    }

    // util method
    public String getOnlineDurarionDesc(Timestamp currentTime) {
        int duration = (int) (currentTime.getTime() - firstRequestTime.getTime());
        return getTimeString(duration);
    }

    public String getDurationSinceLastRequestDesc(Timestamp currentTime) {
        int duration = (int) (currentTime.getTime() - lastRequestTime.getTime());
        return getTimeString(duration);
    }

    public static String getTimeString(int duration) {
        int hours = duration/DateUtil.HOUR;
        int remain = duration - (hours*DateUtil.HOUR);
        int minutes = remain/DateUtil.MINUTE;
        StringBuffer time = new StringBuffer(64);
        if (hours != 0) {
            if (hours == 1) {
                time.append("1 hour and ");
            } else {
                time.append(hours).append(" hours and ");
            }
        }
        if (minutes == 1) {
            time.append("1 minute");
        } else {
            // what if minutes == 0 ???
            time.append(minutes).append(" minute(s)");
        }
        return time.toString();
    }



    /****************************************************************
     * Default package method
     ****************************************************************/
    void setAction(Action action) {
        this.desc = action.getDesc();
        this.url  = action.getUrl();
    }

    /** @todo use method DateUtil.updateCurrentGMTTimestamp() */
    void updateLastRequestTime() {
        DateUtil.updateCurrentGMTTimestamp(lastRequestTime);
    }

    void setMemberID(int memberID) {
        this.memberID = memberID;
    }

    void setMemberName(String memberName) {
        this.memberName = memberName;
    }

}
