/*
 * Copyright (C) 2002 by MyVietnam.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding mvnForum
 * must remain intact in the scripts and in the outputted HTML
 * The "powered by" text/logo with a link back to
 * http://www.mvnForum.com and http://www.MyVietnam.net in the footer of the pages MUST
 * remain visible when the pages are viewed on the internet or intranet.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Support can be obtained from support forums at:
 * http://www.mvnForum.com/mvnforum/index
 *
 * Correspondence and Marketing Questions can be sent to:
 * info@MyVietnam.net
 *
 * @author: Minh Nguyen  minhnn@MyVietnam.net
 * @author: Mai  Nguyen  mai.nh@MyVietnam.net
 */
package net.myvietnam.mvnplugin.mvnforum.user;

import javax.servlet.http.HttpServletRequest;
import net.myvietnam.mvncore.exception.MissingURLMapEntryException;
import net.myvietnam.mvncore.util.StringUtil;
import net.myvietnam.mvnplugin.mvnforum.auth.Action;

public class ActionInUserModule implements Action {

    private String url  = null;
    private String desc = null;

    public ActionInUserModule(HttpServletRequest request, String requestURI) throws MissingURLMapEntryException {
//        lastRequestTime = System.currentTimeMillis();
//        firstRequestTime = lastRequestTime;// be very careful
        url  = null;// url may be null after the code below
        desc = null;// but desc is never be null
        // the request SHOULD ONLY be used to get the queryString
        String queryString = StringUtil.getEmptyStringIfNull(request.getQueryString());

        if (requestURI.equals("/error")) {
            desc = "Have an error [N/A]";
        } else if (requestURI.equals("/index") || requestURI.equals("") || requestURI.equals("/")) {
            url = "index";
            desc = "View forum index";
        } else if (requestURI.equals("/listonlineusers")) {
            url = "listonlineusers";
            desc = "View all online users";
        } else if (requestURI.equals("/listforums")) {
            url = "listforums";
            desc = "View list of all forums";
        } else if (requestURI.equals("/listthreads")) {
            url = "listthreads" + "?" + queryString;
            desc = "View list of topics in forum ???";
        } else if (requestURI.equals("/listrecentthreads")) {
            url = "listrecentthreads" + "?" + queryString;
            desc = "View list of recent topics in all forums";

        } else if (requestURI.equals("/addpost")) {
            //url = "addpost";//xem lai
            desc = "Add new thread or reply to a post [N/A]";// sua lai
        } else if (requestURI.equals("/addpostprocess")) {
            desc = "Have just added a new topic successfully [N/A]";
        } else if (requestURI.equals("/editpost")) {
            //url = "editpost";
            // co the cho coi message do
            desc = "Edit his/her post. You can click here to view that post [N/A]";
        } else if (requestURI.equals("/updatepost")) {
            // co the cho coi message do
            desc = "Have just edited his/her post successfully. You can click here to view that post [N/A]";
        } else if (requestURI.equals("/viewthread")) {
            url = "viewthread" + "?" + queryString;
            desc = "View topic ???";
        } else if (requestURI.equals("/viewmember")) {
            url = "viewmember" + "?" + queryString;
            desc = "View information of member ???";
        } else if (requestURI.equals("/listmembers")) {
            url = "listmembers" + "?" + queryString;
            desc = "View list of members";
        } else if (requestURI.equals("/editmember")) {
            desc = "Edit his/her own information [N/A]";
        } else if (requestURI.equals("/updatemember")) {
            desc = "Have just updated his/her information successfully [N/A]";
        } else if (requestURI.equals("/login")) {
            url = "login";
            desc = "About to login";
        } else if (requestURI.equals("/loginprocess")) {// will be sendRedirect
            desc = "Have just logined successfully [N/A]";
        } else if (requestURI.equals("/logout")) {
            desc = "Have just logouted [N/A]";
        } else if (requestURI.equals("/help")) {
            url = "help";
            desc = "View forum help";
        } else if (requestURI.equals("/search")) {
            url = "search";
            desc = "Search the forum";
        } else if (requestURI.equals("/registermember")) {
            url = "registermember";
            desc = "Sign up to be a member";
        } else if (requestURI.equals("/registermemberprocess")) {
            desc = "Have just signed up successfully [N/A]";
        } else if (requestURI.equals("/myprofile")) {
            desc = "Edit his/her profile [N/A]";
        } else if (requestURI.equals("/changepassword")) {
            desc = "About to change password [N/A]";
        } else if (requestURI.equals("/changepasswordprocess")) {
            desc = "Have just changed password successfully [N/A]";
        } else if (requestURI.equals("/forgotpassword")) {
            url = "forgotpassword";
            desc = "Use forgot password functionality";
        } else if (requestURI.equals("/forgotpasswordprocess")) {
            desc = "Have just received his/her password via email [N/A]";
        } else if (requestURI.equals("/resetpassword")) {
            url = "resetpassword";
            desc = "Use reset password functionality";
        } else if (requestURI.equals("/resetpasswordprocess")) {
            desc = "Have just resetted his/her password [N/A]";
        } else if (requestURI.equals("/changeemail")) {
            desc = "About to change email [N/A]";
        } else if (requestURI.equals("/changeemailprocess")) {
            desc = "Have just changed email successfully [N/A]";
        } else if (requestURI.equals("/changesignature")) {
            desc = "About to change signature [N/A]";
        } else if (requestURI.equals("/changesignatureprocess")) {
            desc = "Have just changed signature successfully [N/A]";
        } else if (requestURI.equals("/changeavatar")) {
            desc = "About to change/upload his/her picture [N/A]";
        } else if (requestURI.equals("/uploadpicture")) {
            desc = "Have just uploaded his/her picture [N/A]";
        } else if (requestURI.equals("/updatepicture")) {
            desc = "Have just changed his/her picture from predefined pictures [N/A]";
        }

        // check that desc is never null
        if (desc == null) {
            String errorMessage = "Cannot find matching entry in OnlineMember for '" + requestURI + "'. Please contact the administrator.";
            throw new MissingURLMapEntryException(errorMessage);
        }
    }

    public String getUrl() {
        return url;
    }

    public String getDesc() {
        return desc;
    }

}
