/*
 * Copyright (C) 2002 by MyVietnam.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * All copyright notices regarding mvnForum
 * must remain intact in the scripts and in the outputted HTML
 * The "powered by" text/logo with a link back to
 * http://www.mvnForum.com and http://www.MyVietnam.net in the footer of the pages MUST
 * remain visible when the pages are viewed on the internet or intranet.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Support can be obtained from support forums at:
 * http://www.mvnForum.com/mvnforum/index
 *
 * Correspondence and Marketing Questions can be sent to:
 * info@MyVietnam.net
 *
 * @author: Minh Nguyen  minhnn@MyVietnam.net
 * @author: Mai  Nguyen  mai.nh@MyVietnam.net
 */
package net.myvietnam.mvnplugin.mvnforum.user;

import javax.servlet.http.*;
import java.sql.*;
import java.util.Collection;
import java.util.Iterator;
import net.myvietnam.mvncore.exception.*;
import net.myvietnam.mvncore.util.ParamUtil;
import net.myvietnam.mvncore.util.DateUtil;
import net.myvietnam.mvncore.filter.DisableHtmlTagFilter;
import net.myvietnam.mvncore.filter.HtmlNewLineFilter;
import net.myvietnam.mvnplugin.mvnforum.db.PostBean;
import net.myvietnam.mvnplugin.mvnforum.db.ThreadBean;
import net.myvietnam.mvnplugin.mvnforum.db.MemberBean;
import net.myvietnam.mvnplugin.mvnforum.MVNForumConfig;
import net.myvietnam.mvnplugin.mvnforum.MyUtil;
import net.myvietnam.mvnplugin.mvnforum.auth.*;

class PostWebHandler {

    private OnlineUserManager userManager = OnlineUserManager.getInstance();

    PostWebHandler() {
    }

    /**
     * This method is for addpost page
     */
    public void prepareAdd(HttpServletRequest request)
        throws DatabaseException, BadInputException, AuthenticationException, AssertionException {

        OnlineUser onlineUser = userManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();

        // we set this action attribute first because the return below can make method return prematurely
        request.setAttribute("action", "addnew");

        int parentPostID    = 0;
        try {
            // neu co parent thi` khong co forum !!!
            parentPostID = ParamUtil.getParameterInt(request, "parent");
        } catch (Exception ex) {
            // do nothing
            // NOTE: we cannot return here since user can have a parameter parent = 0
        }

        if (parentPostID == 0) {// new thread
            int forumID = ParamUtil.getParameterInt(request, "forum");

            permission.ensureCanNewThread(forumID);
        } else {// reply to a post
            // this is a parent post
            PostBean postBean = PostWebHelper.getPost(parentPostID);// can throw DatabaseException

            // check permission
            int forumID  = postBean.getForumID();

            permission.ensureCanNewPost(forumID);

            // now we prepare to list lastest post in the thread
            int threadID = postBean.getThreadID();
            Collection postBeans = PostWebHelper.getLastBeans_inThread_limit(threadID, MVNForumConfig.ROWS_IN_LAST_REPLIES);

            request.setAttribute("ParentPostBean", postBean);
            request.setAttribute("PostBeans", postBeans);
        }
    }

    public void processAdd(HttpServletRequest request)
        throws AssertionException, DatabaseException, CreateException, BadInputException, ForeignKeyNotFoundException, AuthenticationException {

        OnlineUser onlineUser = userManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();

        /** @todo is guest can post thread or message */
        permission.ensureIsAuthenticated();

        int memberID        = onlineUser.getMemberID();
        String memberName   = onlineUser.getMemberName();

        Timestamp now       = DateUtil.getCurrentGMTTimestamp();

        int parentPostID    = ParamUtil.getParameterInt(request, "ParentPostID");

        boolean checkEmpty  = false;
        if (parentPostID == 0) checkEmpty = true;

        String postTopic = ParamUtil.getParameterSafe(request, "PostTopic", checkEmpty);
        String postBody  = ParamUtil.getParameter(request, "message", true);// use message instead of MessageBody
        postBody = DisableHtmlTagFilter.filter(postBody);/** @todo remove this, and use this filter when view */

        int forumID = 0;
        int threadID= 0;
        if (parentPostID == 0) {// new thread
            forumID         = ParamUtil.getParameterInt(request, "ForumID");

            // check permission
            permission.ensureCanNewThread(forumID);

            String lastPostMemberName   = memberName;
            int threadType              = 0;//xem lai
            int threadOption            = 0;//xem lai
            int threadStatus            = 0;//xem lai
            int threadHasPoll           = 0;//xem lai
            String threadIcon           = "";//xem lai
            int threadDuration          = 0;//xem lai
            threadID = ThreadWebHelper.createThread(forumID, memberName, lastPostMemberName,
                                   postTopic, postBody, 0/*threadVoteCount*/,
                                   0/*threadVoteTotalStars*/, now/*threadCreationDate*/, now/*threadLastPostDate*/,
                                   threadType, threadOption, threadStatus,
                                   threadHasPoll, 0/*threadViewCount*/, 0/*threadReplyCount*/,
                                   threadIcon, threadDuration);
        } else {// reply to a post
            PostBean parentPostBean = PostWebHelper.getPost(parentPostID);
            forumID = parentPostBean.getForumID();
            threadID = parentPostBean.getThreadID();

            // check permission
            permission.ensureCanNewPost(forumID);
        }

        Timestamp postLastEditDate = now;
        String postCreationIP       = request.getRemoteAddr();
        String postLastEditIP       = "";
        int postFormatOption        = 0;
        int postOption              = 0;
        int postStatus              = 0;
        String postIcon             = "";
        int postAttachCount         = 0;

        PostWebHelper.createPost(parentPostID, forumID, threadID,
                             memberID, memberName, ""/*lastEditMemberName*/,
                             postTopic, postBody, now/*postCreationDate*/,
                             postLastEditDate, postCreationIP, postLastEditIP,
                             0/*postEditCount*/, postFormatOption, postOption,
                             postStatus, postIcon, postAttachCount);

        MemberWebHelper.increasePostCount(memberID);
        ForumWebHelper.increasePostCount(forumID);
        if (parentPostID == 0) {//new thread, so we increase the ForumThreadCount
            ForumWebHelper.increaseThreadCount(forumID);
        }
        if (parentPostID != 0) {//reply to a post in thread, so we increase the ThreadReplyCount
            ThreadWebHelper.increaseReplyCount(threadID);
        }
        ThreadWebHelper.updateLastPostMemberName(threadID, memberName);
        ForumWebHelper.updateLastPostMemberName(forumID, memberName);
        ForumWebHelper.updateForumLastPostDate(forumID, now);
        ThreadWebHelper.updateThreadLastPostDate(threadID, now);
        /** @todo Update PostEditLog table here */

        request.setAttribute("ForumID", String.valueOf(forumID));
        request.setAttribute("ThreadID", String.valueOf(threadID));
        /**@todo: review, this variable is still reserved*/
        //request.setAttribute("ParentPostID", String.valueOf(parentPostID));
    }

    /**
     * then, it will be forward to addpost.jsp
     * NOTE: This method MUST NOT use parameter MessageParent (need some process to figure out)
     */
    public void prepareEdit(HttpServletRequest request)
        throws DatabaseException, BadInputException, AuthenticationException, AssertionException {

        OnlineUser onlineUser = userManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();

        // a guest CANNOT edit a post, because it need Authenticated Permission
        permission.ensureIsAuthenticated();

        int postID = ParamUtil.getParameterInt(request, "post");
        PostBean postBean = PostWebHelper.getPost(postID);

        int logonMemberID   = onlineUser.getMemberID();
        int authorID        = postBean.getMemberID();

        // check constraint
        if (logonMemberID == authorID) {// same author
            // check date here, usually must not older than 7 days
            Timestamp now = DateUtil.getCurrentGMTTimestamp();
            Timestamp postDate = postBean.getPostCreationDate();
            /** @todo config maxDays */
            int maxDays = 7;
            if ( (now.getTime() - postDate.getTime()) > (DateUtil.DAY * maxDays) ) {
                /** @todo choose a better Exception here */
                throw new BadInputException("You cannot edit a post which is older than " + maxDays + " days.");
            }
            /** @todo check status of this post */
            /*
            if (postBean.getPostStatus() == ?) {
                throw new BadInputException("Cannot edit message which is disable.");
            }*/
        } else {//not an author, so this user must have Edit Permission
            int forumID = postBean.getForumID();

            permission.ensureCanEditPost(forumID);
        }

        request.setAttribute("PostToEdit", postBean);
        request.setAttribute("action", "update");
    }

    /**
     * @todo: log the modification
     * @todo: check the comment below, it's obsolete now :(
     * @todo: check coi messageTopic co the la optional khi reply
     * NOTE: This method MUST NOT get parameter MessageParent (need some process to figure out)
     *      so it needs to call setAttribute with messageParent for page updatepostsuccess.jsp
     */
    public void processUpdate(HttpServletRequest request)
        throws BadInputException, DatabaseException, ForeignKeyNotFoundException, AuthenticationException, AssertionException {

        OnlineUser onlineUser = userManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();

        // a guest CANNOT edit a post, because it need Authenticated Permission
        permission.ensureIsAuthenticated();

        Timestamp now   = DateUtil.getCurrentGMTTimestamp();

        int postID      = ParamUtil.getParameterInt(request, "PostID");// dont change

        // check constraint
        PostBean postBean = PostWebHelper.getPost(postID);
        boolean checkEmpty = true;
        if (postBean.getParentPostID() != 0) {//that is user are editing a REPLY
            checkEmpty = false;
        }
        String postTopic  = ParamUtil.getParameterSafe(request, "PostTopic", checkEmpty);
        String postBody   = ParamUtil.getParameter(request, "message", true);// use message instead of PostBody
        /**@todo: please check it */
        postBody = DisableHtmlTagFilter.filter(postBody);

        int    logonMemberID    = onlineUser.getMemberID();
        String logonMemberName  = onlineUser.getMemberName();
        int    authorID         = postBean.getMemberID();

        // check constraint
        if (logonMemberID == authorID) {// same author
            // check date here, usually must not older than 7 days
            Timestamp postDate = postBean.getPostCreationDate();
            /** @todo config maxDays */
            int maxDays = 7;
            if ( (now.getTime() - postDate.getTime()) > (DateUtil.DAY * maxDays) ) {
                /** @todo choose a better Exception here */
                throw new BadInputException("You cannot edit a post which is older than " + maxDays + " days.");
            }
            /** @todo check status of this post */
            /*
            if (postBean.getPostStatus() == ?) {
                throw new BadInputException("Cannot edit message which is disable.");
            }*/
        } else {//not an author, so this user must have Edit Permission
            int forumID = postBean.getForumID();

            permission.ensureCanEditPost(forumID);
        }

        String postLastEditIP       = request.getRemoteAddr();
        int postFormatOption        = 0;//xem lai
        int postOption              = 0;//xem lai
        int postStatus              = 0;//xem lai
        String postIcon             = "";//xem lai

        /*
         * Note that although the 2 methods below can be combined,
         * I dont do that for clearness
         */
        /** @todo log the modification here */
        PostWebHelper.updatePost(postID, // primary key
                             logonMemberName, postTopic, postBody,
                             now/*postLastEditDate*/, postLastEditIP, postFormatOption,
                             postOption, postStatus, postIcon);
        PostWebHelper.increaseEditCount(postID);

        int threadID = postBean.getThreadID();
        if (postBean.getParentPostID() == 0) {//edit a top post ( thread )
            ThreadWebHelper.updateThreadTopic_Body(threadID, postTopic, postBody);
        }

        request.setAttribute("ForumID", String.valueOf(postBean.getForumID()));
        request.setAttribute("ThreadID", String.valueOf(threadID));
    }

    /**
     * This method is for viewthread page
     */
    /**@todo: change this method's name*/
    public void listPost_inThread(HttpServletRequest request)
        throws DatabaseException, BadInputException, AuthenticationException, AssertionException {

        OnlineUser onlineUser = userManager.getOnlineUser(request);
        MVNForumPermission permission = onlineUser.getPermission();

        int threadID = ParamUtil.getParameterInt(request, "thread");
        ThreadBean threadBean = ThreadWebHelper.getThread(threadID);
        int forumID         = threadBean.getForumID();

        permission.ensureCanReadPost(forumID);
        int offset = 0;
        try {
            offset = ParamUtil.getParameterInt(request, "offset");
        } catch (BadInputException e) {
            // do nothing
        }


        ThreadWebHelper.increaseViewCount(threadID);

        Collection postBeans = PostWebHelper.getPosts_inThread_limit(threadID, offset, MVNForumConfig.ROWS_IN_REPLIES);
        int numberOfPosts = PostWebHelper.getNumberOfPosts_inThread(threadID);

        int length = postBeans.size();
        MemberBean[] memberBeans = new MemberBean[length];
        Iterator iterator = postBeans.iterator();
        int i = 0;
        while(iterator.hasNext()) {
            PostBean bean = (PostBean)iterator.next();
            bean.setPostBody(HtmlNewLineFilter.filter(bean.getPostBody()));
            //very slow here
            /** @todo find a better solution */
            memberBeans[i] = MemberWebHelper.getMember_forPublic(bean.getMemberID());
            i++;
        }

        int previousTopic   = ThreadWebHelper.getPreviousThread(forumID, threadID);// can throw AssertionException
        int nextTopic       = ThreadWebHelper.getNextThread(forumID, threadID);// can throw AssertionException

        MyUtil.prepareNavigate(request, offset, postBeans.size(), numberOfPosts, MVNForumConfig.ROWS_IN_REPLIES);

        request.setAttribute("ThreadBean", threadBean);
        request.setAttribute("PostBeans", postBeans);
        request.setAttribute("MemberBeans", memberBeans);
        request.setAttribute("NumberOfPosts", new Integer(numberOfPosts));
        request.setAttribute("PreviousTopic", new Integer(previousTopic));
        request.setAttribute("NextTopic", new Integer(nextTopic));
        //System.out.println("pre = " + previousTopic + " next = " + nextTopic);
    }

}
